"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Compare the current element hits to the previous snapshot and returns the diff
 */
var GLOBAL_ISTANBUL_CONTAINER_NAMES = ["__coverage__"];
var HitsCollector = /** @class */ (function () {
    function HitsCollector(logger, globalCoverageObject) {
        this.logger = logger;
        this._globalCoverageObject = globalCoverageObject;
        this.latestCoverageSnapshot = {};
    }
    HitsCollector.prototype.getHitElements = function () {
        var hitFilesData = [];
        var globalCoverage = this.getGlobalCoverageObject();
        for (var filename in globalCoverage) {
            var _a = this.getFileCoverageObjects(filename), fileCoverageSnapshot = _a.fileCoverageSnapshot, currentFileCoverage = _a.currentFileCoverage, shouldResolveRelativePath = _a.shouldResolveRelativePath;
            var hitMethods = this.getHitMethods(currentFileCoverage, fileCoverageSnapshot);
            var hitBranches = this.getHitBranches(currentFileCoverage, fileCoverageSnapshot);
            if (hitMethods.length || hitBranches.length) {
                hitFilesData.push({ filename: filename, hitMethods: hitMethods, hitBranches: hitBranches, shouldResolveRelativePath: shouldResolveRelativePath });
            }
        }
        this.updateCoverageSnapshot(globalCoverage);
        return hitFilesData;
    };
    Object.defineProperty(HitsCollector.prototype, "latestCoverageSnapshot", {
        get: function () {
            return this._latestCoverageSnapshot;
        },
        set: function (value) {
            this._latestCoverageSnapshot = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HitsCollector.prototype, "globalCoverageObject", {
        get: function () {
            return this._globalCoverageObject;
        },
        set: function (value) {
            this._globalCoverageObject = value;
        },
        enumerable: true,
        configurable: true
    });
    HitsCollector.prototype.getHitMethods = function (currentHits, snapshotHits) {
        var hitMethodsIndices = Object.keys(currentHits.f).filter(function (id) {
            snapshotHits.f[id] = snapshotHits.f[id] || 0;
            return currentHits.f[id] > snapshotHits.f[id];
        });
        return hitMethodsIndices.map(function (id) { return currentHits.fnMap[id]; });
    };
    HitsCollector.prototype.dropHits = function () {
        this.updateCoverageSnapshot(this.getGlobalCoverageObject());
    };
    HitsCollector.prototype.getHitLeaves = function (currentLeaveHits, snapshotLeaveHits) {
        var hitLeavesIndices = [];
        snapshotLeaveHits = snapshotLeaveHits || Array(currentLeaveHits.length).fill(0);
        currentLeaveHits.forEach(function (leaveHit, leaveIdx) {
            if (currentLeaveHits[leaveIdx] > snapshotLeaveHits[leaveIdx]) {
                hitLeavesIndices.push(leaveIdx);
            }
        });
        return hitLeavesIndices;
    };
    HitsCollector.prototype.getHitBranches = function (currentHits, fileHitSnapshot) {
        var hitBranches = [];
        for (var branchIdx in currentHits.b) {
            var hitLeaves = this.getHitLeaves(currentHits.b[branchIdx], fileHitSnapshot.b[branchIdx]);
            if (hitLeaves.length > 0) {
                var branchData = currentHits.branchMap[branchIdx];
                hitBranches.push({ branchData: branchData, hitLeaves: hitLeaves });
            }
        }
        return hitBranches;
    };
    HitsCollector.prototype.createEmptyIstanbulModule = function (path, useDataKey) {
        if (useDataKey === void 0) { useDataKey = false; }
        var emptyModule = {
            f: {},
            fnMap: {},
            b: {},
            branchMap: {},
            path: path,
            s: {}
        };
        if (useDataKey) {
            return { data: emptyModule };
        }
        return emptyModule;
    };
    HitsCollector.prototype.getFileCoverageObjects = function (filename) {
        var fileCoverageSnapshot = this._latestCoverageSnapshot[filename] || this.createEmptyIstanbulModule(filename);
        var currentFileCoverage = this._globalCoverageObject[filename];
        var shouldResolveRelativePath = true;
        // Istanbul may holds the coverage data under 'data' key see (https://sealights.atlassian.net/browse/SLDEV-4421)
        if (!fileCoverageSnapshot.f && fileCoverageSnapshot.data) {
            fileCoverageSnapshot = fileCoverageSnapshot.data;
            shouldResolveRelativePath = false;
        }
        if (!currentFileCoverage.f && currentFileCoverage.data) {
            currentFileCoverage = currentFileCoverage.data;
        }
        return { fileCoverageSnapshot: fileCoverageSnapshot, currentFileCoverage: currentFileCoverage, shouldResolveRelativePath: shouldResolveRelativePath };
    };
    HitsCollector.prototype.getGlobalCoverageObject = function () {
        if (this._globalCoverageObject)
            return this._globalCoverageObject;
        var re = /^\$\$cov_[0-9]+\$\$$/;
        var keys = Object.keys(global);
        var m;
        for (var i = 0; i < keys.length; i++) {
            var k = keys[i];
            if ((m = re.exec(k)) !== null || GLOBAL_ISTANBUL_CONTAINER_NAMES.indexOf(k) >= 0) {
                this._globalCoverageObject = global[k];
                return this._globalCoverageObject;
            }
        }
        this.logger.warn("Coverage object not found");
        return null;
    };
    HitsCollector.prototype.updateCoverageSnapshot = function (currentCounters) {
        for (var istanbulModule in currentCounters) {
            var useDataKey = currentCounters[istanbulModule].data !== undefined;
            this._latestCoverageSnapshot[istanbulModule] = this._latestCoverageSnapshot[istanbulModule] || this.createEmptyIstanbulModule(istanbulModule, useDataKey);
            var f = currentCounters[istanbulModule].f || currentCounters[istanbulModule].data.f;
            var b = currentCounters[istanbulModule].b || currentCounters[istanbulModule].data.b;
            for (var funcId in f) {
                this.setFunctionHit(istanbulModule, funcId, f, useDataKey);
            }
            for (var branchId in b) {
                this.setBranchHit(b, branchId, istanbulModule, useDataKey);
            }
        }
    };
    HitsCollector.prototype.setBranchHit = function (b, branchId, istanbulModule, useDataKey) {
        var currentBranchesHits = b[branchId];
        this._latestCoverageSnapshot[istanbulModule].b[branchId] = this._latestCoverageSnapshot[istanbulModule].b[branchId] || [];
        for (var i = 0; i < currentBranchesHits.length; i++) {
            if (useDataKey) {
                this._latestCoverageSnapshot[istanbulModule].data.b[branchId][i] = currentBranchesHits[i];
            }
            else {
                this._latestCoverageSnapshot[istanbulModule].b[branchId][i] = currentBranchesHits[i];
            }
        }
    };
    HitsCollector.prototype.setFunctionHit = function (istanbulModule, funcId, f, useDataKey) {
        if (useDataKey) {
            this._latestCoverageSnapshot[istanbulModule].data.f[funcId] = f[funcId];
        }
        else {
            this._latestCoverageSnapshot[istanbulModule].f[funcId] = f[funcId];
        }
    };
    return HitsCollector;
}());
exports.HitsCollector = HitsCollector;
//# sourceMappingURL=hits-collector.js.map